@@ -1,25 +1,31 @@
 package integration
 
 import (
+	"io/fs"
 	"os"
 	"testing"
 
-	"gotest.tools/v3/assert"
-	"gotest.tools/v3/fs"
+	"github.com/stretchr/testify/assert"
+	"github.com/stretchr/testify/require"
+	tfs "gotest.tools/v3/fs"
 )
 
-func setupConfigTest(t *testing.T) *fs.Dir {
-	tmpDir := fs.NewDir(t, "gomplate-inttests",
-		fs.WithDir("indir"),
-		fs.WithDir("outdir"),
-		fs.WithFile(".gomplate.yaml", "in: hello world\n"),
-		fs.WithFile("sleep.sh", "#!/bin/sh\n\nexec sleep $1\n", fs.WithMode(0o755)),
+func setupConfigTest(t *testing.T) *tfs.Dir {
+	t.Helper()
+
+	tmpDir := tfs.NewDir(t, "gomplate-inttests",
+		tfs.WithDir("indir"),
+		tfs.WithDir("outdir"),
+		tfs.WithFile(".gomplate.yaml", "in: hello world\n"),
+		tfs.WithFile("sleep.sh", "#!/bin/sh\n\nexec sleep $1\n", tfs.WithMode(0o755)),
 	)
 	t.Cleanup(tmpDir.Remove)
 	return tmpDir
 }
 
-func writeFile(t *testing.T, dir *fs.Dir, f, content string) {
+func writeFile(t *testing.T, dir *tfs.Dir, f, content string) {
+	t.Helper()
+
 	f = dir.Join(f)
 	err := os.WriteFile(f, []byte(content), 0o600)
 	if err != nil {
@@ -27,7 +33,7 @@ func writeFile(t *testing.T, dir *fs.Dir, f, content string) {
 	}
 }
 
-func writeConfig(t *testing.T, dir *fs.Dir, content string) {
+func writeConfig(t *testing.T, dir *tfs.Dir, content string) {
 	t.Helper()
 
 	writeFile(t, dir, ".gomplate.yaml", content)
@@ -97,7 +103,7 @@ datasources:
 	assertSuccess(t, o, e, err, "")
 
 	b, err := os.ReadFile(tmpDir.Join("outdir", "file"))
-	assert.NilError(t, err)
+	require.NoError(t, err)
 	assert.Equal(t, "hello world", string(b))
 }
 
@@ -123,7 +129,7 @@ outputFiles: [out]
 	assertSuccess(t, o, e, err, "")
 
 	b, err := os.ReadFile(tmpDir.Join("out"))
-	assert.NilError(t, err)
+	require.NoError(t, err)
 	assert.Equal(t, "hello world", string(b))
 }
 
@@ -165,7 +171,7 @@ datasources:
 
 	o, e, err := cmd(t).withDir(tmpDir.Path()).
 		withEnv("GOMPLATE_LEFT_DELIM", "<<").run()
-	assert.NilError(t, err)
+	require.NoError(t, err)
 	assert.Equal(t, "", e)
 	assert.Equal(t, "hello world", o)
 }
@@ -189,7 +195,7 @@ datasources:
 	o, e, err := cmd(t, "--left-delim={{").
 		withDir(tmpDir.Path()).
 		withEnv("GOMPLATE_LEFT_DELIM", "<<").run()
-	assert.NilError(t, err)
+	require.NoError(t, err)
 	assert.Equal(t, "", e)
 	assert.Equal(t, "hello world", o)
 }
@@ -227,8 +233,68 @@ suppressEmpty: true
 
 	_, _, err := cmd(t).withDir(tmpDir.Path()).
 		withEnv("GOMPLATE_SUPPRESS_EMPTY", "false").run()
-	assert.NilError(t, err)
+	require.NoError(t, err)
 
 	_, err = os.Stat(tmpDir.Join("missing"))
-	assert.Equal(t, true, os.IsNotExist(err))
+	require.ErrorIs(t, err, fs.ErrNotExist)
+}
+
+func TestConfig_ConfigParseErrorSpecifiesFilename(t *testing.T) {
+	tmpDir := setupConfigTest(t)
+
+	writeConfig(t, tmpDir, `templates:
+  dir: /foo/bar
+`)
+	_, _, err := cmd(t).withDir(tmpDir.Path()).run()
+	assert.ErrorContains(t, err, `parsing config file ".gomplate.yaml": YAML decoding failed`)
+}
+
+func TestConfig_ConfigTemplatesSupportsMap(t *testing.T) {
+	tmpDir := setupConfigTest(t)
+
+	writeConfig(t, tmpDir, `in: '{{ template "t1" (dict "testValue" "12345") }}'
+templates:
+  t1:
+    url: t1.tmpl
+`)
+	writeFile(t, tmpDir, "t1.tmpl", `{{ .testValue }}`)
+
+	o, e, err := cmd(t).withDir(tmpDir.Path()).run()
+	assertSuccess(t, o, e, err, "12345")
+}
+
+func TestConfig_ConfigTemplatesSupportsArray(t *testing.T) {
+	tmpDir := setupConfigTest(t)
+
+	// TODO: remove this test once the array format is no longer supported
+	writeConfig(t, tmpDir, `in: '{{ template "t1" (dict "testValue" "12345") }}'
+templates:
+  - t1=t1.tmpl
+`)
+	writeFile(t, tmpDir, "t1.tmpl", `{{ .testValue }}`)
+
+	o, e, err := cmd(t).withDir(tmpDir.Path()).run()
+	assert.Contains(t, e, "Deprecated: config: the YAML array form for 'templates' is deprecated")
+	assert.Equal(t, "12345", o)
+	require.NoError(t, err)
+}
+
+func TestConfig_MissingKeyDefault(t *testing.T) {
+	tmpDir := setupConfigTest(t)
+	writeConfig(t, tmpDir, `inputFiles: [in]
+missingKey: default
+`)
+	writeFile(t, tmpDir, "in", `{{ .name }}`)
+
+	o, e, err := cmd(t).withDir(tmpDir.Path()).run()
+	assertSuccess(t, o, e, err, `<no value>`)
+}
+
+func TestConfig_MissingKeyNotDefined(t *testing.T) {
+	tmpDir := setupConfigTest(t)
+	writeConfig(t, tmpDir, `inputFiles: [in]`)
+	writeFile(t, tmpDir, "in", `{{ .name }}`)
+
+	o, e, err := cmd(t).withDir(tmpDir.Path()).run()
+	assertFailed(t, o, e, err, `map has no entry for key \"name\"`)
 }
