@@ -15,50 +15,64 @@ import (
 	"testing"
 	"time"
 
-	gcmd "github.com/hairyhenderson/gomplate/v3/internal/cmd"
+	gcmd "github.com/hairyhenderson/gomplate/v4/internal/cmd"
 	vaultapi "github.com/hashicorp/vault/api"
 	"github.com/stretchr/testify/assert"
+	"github.com/stretchr/testify/require"
 	"gotest.tools/v3/icmd"
 )
 
 const isWindows = runtime.GOOS == "windows"
 
 // a convenience...
-func inOutTest(t *testing.T, i, o string) {
+func inOutTest(t *testing.T, i, o string, args ...string) {
 	t.Helper()
 
-	stdout, stderr, err := cmd(t, "-i", i).run()
-	assert.NoError(t, err)
-	assert.Equal(t, "", stderr)
-	assert.Equal(t, o, stdout)
+	args = append(args, "-i", i)
+	stdout, stderr, err := cmd(t, args...).run()
+	assertSuccess(t, stdout, stderr, err, o)
+}
+
+func inOutContainsError(t *testing.T, i, e string, args ...string) {
+	t.Helper()
+
+	args = append(args, "-i", i)
+	stdout, stderr, err := cmd(t, args...).run()
+	assertFailed(t, stdout, stderr, err, e)
 }
 
 func inOutTestExperimental(t *testing.T, i, o string) {
 	t.Helper()
 
 	stdout, stderr, err := cmd(t, "--experimental", "-i", i).run()
-	assert.NoError(t, err)
-	assert.Equal(t, "", stderr)
-	assert.Equal(t, o, stdout)
+	assertSuccess(t, stdout, stderr, err, o)
 }
 
 func inOutContains(t *testing.T, i, o string) {
 	t.Helper()
 
 	stdout, stderr, err := cmd(t, "-i", i).run()
-	assert.NoError(t, err)
 	assert.Equal(t, "", stderr)
 	assert.Contains(t, stdout, o)
+	require.NoError(t, err)
 }
 
 func assertSuccess(t *testing.T, o, e string, err error, expected string) {
 	t.Helper()
 
-	assert.NoError(t, err)
+	require.NoError(t, err)
 	assert.Equal(t, "", e)
 	assert.Equal(t, expected, o)
 }
 
+func assertFailed(t *testing.T, o, e string, err error, expected string) {
+	t.Helper()
+
+	assert.Contains(t, e, expected)
+	assert.Equal(t, "", o)
+	require.Error(t, err)
+}
+
 // mirrorHandler - reflects back the HTTP headers from the request
 func mirrorHandler(w http.ResponseWriter, r *http.Request) {
 	type Req struct {
@@ -75,12 +89,24 @@ func mirrorHandler(w http.ResponseWriter, r *http.Request) {
 }
 
 func typeHandler(t, body string) func(http.ResponseWriter, *http.Request) {
-	return func(w http.ResponseWriter, r *http.Request) {
+	return func(w http.ResponseWriter, _ *http.Request) {
 		w.Header().Set("Content-Type", t)
 		w.Write([]byte(body))
 	}
 }
 
+func paramHandler(t *testing.T) http.HandlerFunc {
+	return func(w http.ResponseWriter, r *http.Request) {
+		// just returns params as JSON
+		w.Header().Set("Content-Type", "application/json")
+
+		enc := json.NewEncoder(w)
+		if err := enc.Encode(r.URL.Query()); err != nil {
+			t.Fatalf("error encoding: %v", err)
+		}
+	}
+}
+
 // freeport - find a free TCP port for immediate use. No guarantees!
 func freeport(t *testing.T) (port int, addr string) {
 	l, err := net.ListenTCP("tcp", &net.TCPAddr{IP: net.ParseIP("127.0.0.1")})
@@ -95,12 +121,20 @@ func freeport(t *testing.T) (port int, addr string) {
 
 // waitForURL - waits up to 20s for a given URL to respond with a 200
 func waitForURL(t *testing.T, url string) error {
+	ctx, cancel := context.WithTimeout(context.Background(), 20*time.Second)
+	defer cancel()
+
 	client := http.DefaultClient
 	retries := 100
 	for retries > 0 {
 		retries--
 		time.Sleep(200 * time.Millisecond)
-		resp, err := client.Get(url)
+		req, err := http.NewRequestWithContext(ctx, http.MethodGet, url, nil)
+		if err != nil {
+			return err
+		}
+
+		resp, err := client.Do(req)
 		if err != nil {
 			t.Logf("Got error, retries left: %d (error: %v)", retries, err)
 			continue
@@ -193,6 +227,8 @@ func (c *command) withEnv(k, v string) *command {
 var GomplateBinPath = ""
 
 func (c *command) run() (o, e string, err error) {
+	c.t.Helper()
+
 	if GomplateBinPath != "" {
 		return c.runCompiled(GomplateBinPath)
 	}
@@ -200,6 +236,8 @@ func (c *command) run() (o, e string, err error) {
 }
 
 func (c *command) runInProcess() (o, e string, err error) {
+	c.t.Helper()
+
 	// iterate env vars by order of insertion
 	for _, k := range c.envK {
 		k := k
@@ -224,11 +262,15 @@ func (c *command) runInProcess() (o, e string, err error) {
 		if err != nil {
 			c.t.Fatal(err)
 		}
+
+		c.t.Logf("running in dir %q", c.dir)
 	}
 
 	stdin := strings.NewReader(c.stdin)
 
-	ctx := context.Background()
+	ctx, cancel := context.WithCancel(context.Background())
+	defer cancel()
+
 	stdout, stderr := &bytes.Buffer{}, &bytes.Buffer{}
 	err = gcmd.Main(ctx, c.args, stdin, stdout, stderr)
 	return stdout.String(), stderr.String(), err
